/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb;

import java.io.IOException;
import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.DefaultHandler;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.Iterator;

class ArgsReader extends DefaultHandler
  {
  private HashMap pActivityMap = null;
  private String curPAName = "all";

  Vector uris = new Vector();
  Vector argPMethods = new Vector();

  boolean displayFlag = false;
  boolean verboseFlag = false;
  boolean gcFlag = false;
  boolean gcMeasuredFlag = false;
  int loopCount = 5;

  ArgsReader(HashMap map)
    {
	pActivityMap = map;
    }

  public void startElement(String uri, String localname, String name, Attributes attrs)
	{
	if (name.equalsIgnoreCase("Flags"))
	  {
	  gcFlag = Boolean.valueOf(attrs.getValue("gc")).booleanValue();
	  gcMeasuredFlag = Boolean.valueOf(attrs.getValue("gcMeasured")).booleanValue();
	  verboseFlag = Boolean.valueOf(attrs.getValue("verbose")).booleanValue();
	  displayFlag = Boolean.valueOf(attrs.getValue("display")).booleanValue();
	  }
	else if (name.equalsIgnoreCase("Params"))
	  {
	  try
	    {
	    if (attrs.getValue("loopCount") != null)
	    loopCount = Integer.parseInt(attrs.getValue("loopCount"));
	    }
      catch (NumberFormatException e)
		{
	    System.err.println("Invalid Loop Count specified. Exception: " + e);
		System.err.println("Resorting to default value of: " + loopCount);
		}
	  }
	else if (name.equalsIgnoreCase("Target"))
	  {
	  String target = attrs.getValue("name");
	  if (target != null)
	    uris.add(target);
	  }
	else if (name.equalsIgnoreCase("ProcessingActivity"))
      {
	  String paName = attrs.getValue("name");

	  if (!(paName.equals("all") || pActivityMap.containsKey(paName)))
	    System.err.println("Unrecognised ProcessingActivity: " + paName);
      curPAName = paName;
	  }
	else if (name.equalsIgnoreCase("ProcessingMethod"))
      {
	  String mName = attrs.getValue("name");
	  String enabled = attrs.getValue("enabled");
	  if (!Boolean.valueOf(enabled).booleanValue())		// Not enabled. Skip.
	    return;

	  PMethodIterator itr = new PMethodIterator(pActivityMap, curPAName);
	  boolean found = false;
	  while (itr.hasNext())
		{
		PMethod pm = (PMethod)itr.next();
		if (mName.equals("all") || mName.equals(pm.name))
		  {
		  argPMethods.add(pm);
		  found = true;
		  }
		}
	  if (!found)	// No match found.
		System.err.println("Unrecognised method: " + mName);
	  }
	} // startElement()

  void displayArgs()
    {
	System.out.println("----------------------- XMLBench 1.0 ----------------");
	System.out.println("Garbage Collection Flag  : " + gcFlag);
	System.out.println("GC in measurement Window : " + gcMeasuredFlag);
	System.out.println("Iteration loop count     : " + loopCount);
	System.out.println("Show Performance Metrics : " + displayFlag);
	System.out.print("Enabled (PActivity, PMethod) tuples:");
	for (int k = 0; k < argPMethods.size(); k++)
	  {
	  PMethod pm = (PMethod)argPMethods.elementAt(k);
	  System.out.print(" (" + pm.paName + ", " + pm.name + ")");
      }
    System.out.print("\n");
	System.out.print("Run URIs:");
	for (int k = 0; k < uris.size(); k++)
	  System.out.print(" " + (String)uris.elementAt(k));
	System.out.print("\n");
	System.out.println("-----------------------------------------------------");
    }

  void readArgs()
    {
	try
	  {
	  SAXParserFactory factory = SAXParserFactory.newInstance();
	  SAXParser p = factory.newSAXParser();
	  p.parse("args.xml", this);
	  }
    catch (ParserConfigurationException e)
	  {
	  System.err.println("ArgsReader:: Failed to get a SAXParser " + e.getMessage());
	  System.exit(1);
	  }
	catch (SAXException e)
	  {
	  System.err.println("ArgsReader:: SAX error: " + e.getMessage());
	  System.exit(1);
	  }
	catch (IOException e)
	  {
	  System.err.println("ArgsReader:: I/O error: " + e.getMessage());
	  System.exit(1);
	  }
	}
  }
