/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb;

import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import org.xml.sax.helpers.DefaultHandler;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.Iterator;

public class ConfReader extends DefaultHandler
  {
  private HashMap pActivityMap = null;

  private PActivity curPActivity = null;
  private PMethod curPMethod = null;


  public void startElement(String uri, String localname, String name, Attributes attrs)
	{
	if (name.equalsIgnoreCase("xpbconf"))
	  {
	  pActivityMap = null;
	  curPActivity = null;
	  curPMethod = null;
	  }
	else if (name.equalsIgnoreCase("ProcessingActivities"))
	  {
	  if (pActivityMap != null)		// No ActivityMap already initialized ==> error
	    configError(name);
	  pActivityMap = new HashMap();
	  }
	else if (name.equalsIgnoreCase("ProcessingActivity"))
      {
	  String procName = attrs.getValue("name");
	  if (pActivityMap == null || procName == null)
	    configError(name);
	  curPActivity = new PActivity(procName);
	  pActivityMap.put(procName, curPActivity);
	  }
	else if (name.equalsIgnoreCase("ProcessingMethods"))
      {
	  if (curPActivity == null)		// No ProcessingActivity specified ==> error
	    configError(name);
	  }
	else if (name.equalsIgnoreCase("ProcessingMethod"))
      {
	  String methodName = attrs.getValue("name");
	  String className = attrs.getValue("class");
	  if (curPActivity == null || methodName == null || className == null)
	    configError(name);
	  curPMethod = new PMethod(methodName, curPActivity.name, className);
	  curPActivity.methods.put(methodName, curPMethod);
	  }
	else if (name.equalsIgnoreCase("Params"))
      {
	  if (curPMethod == null)
	    configError(name);
	  curPMethod.args = new HashMap();
	  }
	else if (name.equalsIgnoreCase("Param"))
      {
	  if (curPMethod == null || curPMethod.args == null)
	    configError(name);
	  String pname = attrs.getValue("name");
	  String pvalue = attrs.getValue("value");
	  curPMethod.args.put(pname, pvalue);
	  }
	} // startElement()

  public HashMap configure()
    {
	try
	  {
	  SAXParserFactory factory = SAXParserFactory.newInstance();
	  SAXParser p = factory.newSAXParser();
	  p.parse("conf.xml", this);
	  }
    catch (ParserConfigurationException e)
	  {
	  System.err.println("ConfReader:: Failed to get a SAXParser " + e.getMessage());
	  System.exit(1);
	  }
	catch (SAXException e)
	  {
	  System.err.println("ConfReader:: SAX error: " + e.getMessage());
	  System.exit(1);
	  }
	catch (IOException e)
	  {
	  System.err.println("ConfReader:: I/O error: " + e.getMessage());
	  System.exit(1);
	  }
	return pActivityMap;
	}

  private void configError(String name)
    {
    System.err.println("Configuration Error at element: " + name);
    System.exit(1);
	}
  }
