/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb;

import java.util.Map;
import java.util.HashMap;
import java.util.Vector;
import java.util.Iterator;

class PMethod
  {
  String name;
  String paName;
  String className;
  Processor processor;
  HashMap args;
  long[] execTime;
  long[] memUsed;
  Result[] res;

  PMethod(String name, String paName, String className)
	{
	this.name = name;
	this.paName = paName;
	this.className = className;
	args = null;
	execTime = null;
	memUsed = null;
	res = null;
    }
  void process(int loopCount, Vector uris, boolean gcFlag, boolean gcMeasuredFlag) throws XPBException
    {
	Runtime rt = Runtime.getRuntime();
    execTime = new long[loopCount];
    memUsed = new long[loopCount];
    res = new Result[uris.size()];

	for (int i = 0; i < loopCount; i++)
	  {
	  if (gcFlag)
	    rt.gc();		// force garbage collection.
	  long sm1 = rt.totalMemory() - rt.freeMemory();

	  long st = System.currentTimeMillis();
	  for (int k = 0; k < uris.size(); k++)
	    res[k] = processor.process((String)uris.elementAt(k));
	  long sm2 = rt.totalMemory() - rt.freeMemory();
	  if (gcMeasuredFlag)
	    rt.gc();		// force garbage collection.
	  long et = System.currentTimeMillis();

	  execTime[i] = et - st;
	  memUsed[i] = sm2 - sm1;
      }
    }

  String perfDataAsXML()
    {
	StringBuffer sb = new StringBuffer();
	sb.append("    <PerfDataItem>\n");
	sb.append("      <ProcessingActivity name=\"" + paName + "\"/>\n");
	sb.append("      <ProcessingMethod name=\"" + name + "\"/>\n");
	if (args != null)
	  {
	  sb.append("      <Params>\n");
	  Iterator itr = args.entrySet().iterator();
	  while (itr.hasNext())
	    {
		Map.Entry entry = (Map.Entry)itr.next();
		sb.append("        <Param name=\"" + entry.getKey() + "\" value=\"" + entry.getValue() + "\"/>\n");
		}
	  sb.append("      </Params>\n");
	  }
	sb.append("      <ExecutionRuns timeunits=\"millis\" memunits=\"KB\"/>\n");
	for (int i = 0; i < execTime.length; i++)
	  {
	  sb.append("        <ExecutionRun run=\"" + i + "\" exectime=\"" + execTime[i] + "\" memused=\""
	       + (memUsed[i]/1024) + "\"/>\n");
	  }
	sb.append("      </ExecutionRuns>\n");
	sb.append("    </PerfDataItem>\n");
	return sb.toString();
	}

  String resultsAsXML(Vector uris)
    {
	StringBuffer sb = new StringBuffer();

	sb.append("    <ProcessingActivity name=\"" + paName + "\"/>\n");
	sb.append("    <ProcessingMethod name=\"" + name + "\"/>\n");
	for (int k = 0; k < uris.size(); k++)
	  {
	  sb.append("    <ProcessingResult>\n");
	  sb.append("      <Target name=\"" + uris.elementAt(k) + "\"/>\n");
	  sb.append(res[k].resultsAsXML());
	  sb.append("\n");
	  sb.append("    </ProcessingResult>\n");
	  }

	return sb.toString();
	}
  }
