/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat;

import java.io.*;

import java.util.Iterator;

import java.util.TreeMap;
import java.util.HashMap;
import java.util.ArrayList;
import org.xperf.xpb.Result;

public class XMLStat implements Result
  {
  static class Elinfo
    {
	public String name;
	public int count;
	public int minlev;
	public int charcount;
	public int seen;
	public boolean empty;
	public TreeMap ptab;
	public TreeMap ktab;
	public TreeMap atab;

	public Elinfo(String nm)
	  {
	  name = nm;
	  count = minlev = charcount = 0;
	  seen = 0;
	  empty = true;
	  ptab = new TreeMap();
	  ktab = new TreeMap();
	  atab = new TreeMap();
	  }
    }

  static class Int
    {
    public int val;

    public Int (int i)
	  {
      val = i;
      }
    }

  HashMap eltab = new HashMap(128);

  int global_seen = 0;
  String root;

  static class ElComp implements java.util.Comparator
    {
    public int compare(Object ao, Object bo)
	  {
	  Elinfo a = (Elinfo) ao;
	  Elinfo b = (Elinfo) bo;
	  int c1 = a.minlev - b.minlev;
	  if (c1 != 0)
	    return c1;
	  return a.seen - b.seen;
	  }

    public boolean equals(Object ao, Object bo)
	  {
	  Elinfo a = (Elinfo) ao;
	  Elinfo b = (Elinfo) bo;
	  return ((a.minlev == b.minlev) && (a.seen == b.seen));
	  }
    }

  void set_minlev(String name, int level)
    {
	Elinfo inf = (Elinfo) eltab.get(name);

	if (inf.minlev == 0 || inf.minlev > level)
	  {
	  int newlev = level + 1;
	  inf.minlev = level;
	  Iterator iter = inf.ktab.keySet().iterator();
	  while (iter.hasNext())
	    {
        String kname = (String) iter.next();
		set_minlev(kname, newlev);
	    }
	  }
    }

  public void setRoot(String name)
    {
	root = name;
    }

  public void addElem(String ename)
    {
    Elinfo inf = (Elinfo)eltab.get(ename);
	if (inf == null)
	  {
	  inf = new Elinfo(ename);
	  inf.seen = global_seen++;
	  eltab.put(ename, inf);
	  }

	inf.count++;
	}

  public void incParentCount(String ename, String parent)
    {
	Elinfo inf = (Elinfo)eltab.get(ename);
	Int pcnt = (Int) inf.ptab.get(parent);
	if (pcnt == null)
	  inf.ptab.put(parent, new Int(1));
	else
	  pcnt.val++;
    }

  public void incChildCount(String ename, String child)
    {
	Elinfo pinf = (Elinfo)eltab.get(ename);
	Int kcnt = (Int) pinf.ktab.get(child);
	if (kcnt == null)
	  pinf.ktab.put(child, new Int(1));
	else
	  kcnt.val++;
	pinf.empty = false;
    }

  public void incAttrCount(String ename, String attnm)
    {
	Elinfo inf = (Elinfo)eltab.get(ename);
	Int acnt = (Int) inf.atab.get(attnm);
   	if (acnt == null)
	  inf.atab.put(attnm, new Int(1));
	else
	  acnt.val++;
    }

  public void addCharCount(String ename, int len)
    {
    Elinfo inf = (Elinfo) eltab.get(ename);
	inf.empty = false;
	inf.charcount += len;
    }

  void showtab(String label, TreeMap tab, boolean dosum)
    {
    if (tab.size() == 0)
      return;
    System.out.println("\n   " + label + ":");
    int sum = 0;

    Iterator step = tab.keySet().iterator();
    while (step.hasNext())
	  {
	  String name = (String) step.next();
	  int cnt = ((Int) tab.get(name)).val;
	  sum += cnt;
	  System.out.print("      ");
	  System.out.print(name);
	  String cstr = Integer.toString(cnt);
	  int lim = 22 - name.length() + 5 - cstr.length();
	  int i;
	  for (i = 0; i < lim; i++)
	  System.out.print(" ");
	  System.out.println(cstr);
	  }

	if (dosum && tab.size() > 1)
	  {
	  System.out.println("                            =====");
	  String sstr = Integer.toString(sum);
	  int lim = 33 - sstr.length();
	  int i;
	  for (i = 0; i < lim; i++)
        System.out.print(" ");
	  System.out.println(sstr);
	  }
    }
  public void show()
    {
  	set_minlev(root, 1);

  	ArrayList sorted = new ArrayList(eltab.values());
  	java.util.Collections.sort(sorted, new ElComp());
  	Iterator step = sorted.iterator();

  	while (step.hasNext())
  	  {
  	  Elinfo inf = (Elinfo) step.next();
  	  System.out.println("\n================\n" + inf.name
  			       + ": " + inf.count);
  	  if (inf.charcount > 0)
  		System.out.println("Had " + inf.charcount
  				   + " bytes of character data");
  	  if (inf.empty)
  		System.out.println("Always empty");

  	  showtab("Parents", inf.ptab, false);
  	  showtab("Children", inf.ktab, true);
  	  showtab("Attributes", inf.atab, false);
	  }
    }

  String tabAsXML(String label1, String label2, TreeMap tab, boolean dosum)
    {
    if (tab.size() == 0)
      return "";

    int sum = 0;
    StringBuffer sb = new StringBuffer();
    Iterator step = tab.keySet().iterator();
    while (step.hasNext())
	  {
	  String name = (String) step.next();
	  int cnt = ((Int) tab.get(name)).val;
	  sum += cnt;
	  sb.append("            <" + label2 + " name=\"" + name + "\" count=\"" + cnt + "\"/>\n");
	  }

	StringBuffer sb1 = new StringBuffer();
    sb1.append("          <" + label1);
	if (dosum && tab.size() > 1)
	  sb1.append(" sum=\"" + sum + "\"");
	sb1.append(">\n");
	sb1.append(sb.toString());

	sb1.append("          </" + label1 + ">\n");
	return sb1.toString();
    }

  public String resultsAsXML()
    {
	StringBuffer sb = new StringBuffer();
  	set_minlev(root, 1);

  	ArrayList sorted = new ArrayList(eltab.values());
  	java.util.Collections.sort(sorted, new ElComp());
  	Iterator step = sorted.iterator();

    int elemCount = 0;
  	while (step.hasNext())
  	  {
  	  Elinfo inf = (Elinfo) step.next();
  	  sb.append("        <Element name=\"" + inf.name + "\" count=\"" + inf.count + "\" chardata=\"");
  	  if (inf.charcount > 0)
  		sb.append(inf.charcount);
  	  if (inf.empty)
  		sb.append("empty");
      sb.append("\">\n");

  	  sb.append(tabAsXML("Parents", "Parent", inf.ptab, false));
  	  sb.append(tabAsXML("Children", "Child", inf.ktab, true));
  	  sb.append(tabAsXML("Attributes", "Attribute", inf.atab, false));

  	  sb.append("        </Element>\n");
  	  ++elemCount;
	  }

	StringBuffer sb1 = new StringBuffer();
	sb1.append("      <XStatResult xmlns=\"http://www.xperf.org/xpb/xstat\" elements=\"" + elemCount + "\">\n");
	sb1.append(sb);
	sb1.append("      </XStatResult>\n");
	return sb1.toString();
	}
  }
