/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat.dom;

import org.xperf.xpb.xstat.XMLStat;
import org.xperf.xpb.Processor;
import org.xperf.xpb.Result;
import org.xperf.xpb.XPBException;

import java.io.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;

import java.util.HashMap;

public class DOMProcessor implements Processor
  {
  public Result process(String xmlFile) throws XPBException
    {
	XMLStat stat = new XMLStat();
    Document doc = null;
	DocumentBuilder b = null;
	try
	  {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	  b = factory.newDocumentBuilder();
      }
	catch (ParserConfigurationException e)
	  {
	  System.err.println("Failed to get a SAXParser " + e.getMessage());
	  System.exit(1);
	  }


    StatCollector statCollector = new StatCollector(stat);
	try
	  {
	  doc = b.parse(xmlFile);
	  Node root = doc.getDocumentElement();
	  collectStat(root, statCollector);
	  }
	catch (FileNotFoundException e)
	  {
	  throw new XPBException("DOMProcessor:: File not found. Exception: " + e);
	  }
	catch (SAXException e)
	  {
	  throw new XPBException("DOMProcessor:: SAX parsing error. Exception: " + e);
	  }
	catch (IOException e)
	  {
	  throw new XPBException("DOMProcessor:: I/O error. Exception: " + e);
	  }
	return stat;
    }
  public void   setup(HashMap map)
    {
	}
  private void collectStat(Node n, StatCollector sc)
    {
	if (n.getNodeType() == Node.ELEMENT_NODE)
	  {
	  sc.element((Element)n);
	  NodeList children = n.getChildNodes();
	  for (int i = 0; i < children.getLength(); i++)
	    collectStat((Node)children.item(i), sc);
      }
	else if (n.getNodeType() == Node.TEXT_NODE)
	  sc.text((Text)n);
	}
  }
