/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat.pull;

import org.xperf.xpb.xstat.XMLStat;
import org.xperf.xpb.Processor;
import org.xperf.xpb.Result;
import org.xperf.xpb.XPBException;

import java.io.*;
import org.gjt.xpp.*;
import java.util.HashMap;

public class PullProcessor implements Processor
  {
  public Result process(String xmlFile) throws XPBException
    {
	XMLStat stat = new XMLStat();
	XmlPullParser p = null;

	XmlPullParserFactory factory = null;
    XmlStartTag stag = null;
    XmlEndTag etag = null;
	try
	  {
	  factory = XmlPullParserFactory.newInstance();
      stag = factory.newStartTag();
      etag = factory.newEndTag();
	  p = factory.newPullParser();
      }
	catch (XmlPullParserException e)
	  {
	  throw new XPBException("PullProcessor:: Failed to get a XmlPullParser. Exception: " + e.getMessage());
	  }

    StatCollector statCollector = new StatCollector(stat);

	try
	  {
	  Reader reader = new BufferedReader(new FileReader(xmlFile));
	  //Reader reader = new FileReader(xmlFile);
	  p.setInput(reader);

      byte type;
	  while ((type = p.next()) != XmlPullParser.END_DOCUMENT)
	    {
	    if(type == XmlPullParser.CONTENT)
	      {
		  String s = p.readContent();
		  statCollector.content(s.length());
	      }
	    else if(type == XmlPullParser.END_TAG)
	      {
		  p.readEndTag(etag);
		  statCollector.endElement(etag);
	      }
	    else if(type == XmlPullParser.START_TAG)
	      {
		  p.readStartTag(stag);
		  statCollector.startElement(stag);
	      }
	    } //while
	  } //try
	catch (XmlPullParserException e)
	  {
	  throw new XPBException("PullProcessor:: Parsing failed. Exception: " + e.getMessage());
	  }
	catch (FileNotFoundException e)
	  {
	  throw new XPBException("PullProcessor:: File not found. Exception: " + e.getMessage());
	  }
	catch (IOException e)
	  {
	  throw new XPBException("PullProcessor:: I/O Error. Exception: " + e.getMessage());
	  }
	return stat;
    }
  public void   setup(HashMap map)
    {
	}
  }
