/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat.pull;

import org.xperf.xpb.xstat.XMLStat;
import org.gjt.xpp.*;
import java.util.*;

public class StatCollector
  {
  private XMLStat stat = null;
  private Stack elstack = new Stack();

  public StatCollector(XMLStat stat)
    {
	this.stat = stat;
    }

  public void startElement(XmlStartTag stag)
	{
	String name = stag.getLocalName();
	stat.addElem(name);

	try
	  {
	  String parent = (String)elstack.peek();
	  stat.incParentCount(name, parent);
      stat.incChildCount(parent, name);
	  }
	catch (EmptyStackException e)
	  {
	  stat.setRoot(name);
	  }

	// Attribute handling

	int n = stag.getAttributeCount();
    for (int i = 0; i < n; i++)
      {
      String attnm = stag.getAttributeRawName(i);
      stat.incAttrCount(name, attnm);
      }

	elstack.push(name);
	}

  public void endElement(XmlEndTag etag)
	{
	try
	  {
	  elstack.pop();
	  }
	catch (EmptyStackException e)
	  {
	  System.err.println("xpb.xstat.pull.StatCollector:: Tried to pop empty element stack");
	  }
	}

  public void content(int len)
	{
	String elname = null;
	try
	  {
	  elname = (String)elstack.peek();
	  }
    catch (EmptyStackException e)
      {
	  System.err.println("xpb.xstat.pull.StatCollector:: No element on stack");
	  }
    stat.addCharCount(elname, len);
	}
  }

