/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat.sax;

import org.xperf.xpb.xstat.XMLStat;
import org.xperf.xpb.Processor;
import org.xperf.xpb.Result;
import org.xperf.xpb.XPBException;

import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import java.util.HashMap;


public class SAXProcessor implements Processor
  {
  final static String SAXParserFactoryProp = "javax.xml.parsers.SAXParserFactory";
  private HashMap params = null;

  public Result process(String xmlFile) throws XPBException
    {
	XMLStat stat = new XMLStat();

    if (params != null)
      {
	  String factoryImpl = (String)params.get(SAXParserFactoryProp);
	  if (params.get(SAXParserFactoryProp) != null)
	    System.setProperty(SAXParserFactoryProp, factoryImpl);
	  }
	/* Doesn't work with Xerces. Works fine with Crimson.
	else
     System.setProperty(SAXParserFactoryProp, "");
    */

    SAXParserFactory factory = SAXParserFactory.newInstance();
	SAXParser p = null;
	try
	  {
	  p = factory.newSAXParser();
      }
	catch (ParserConfigurationException e)
	  {
	  throw new XPBException("SAXProcessor:: Failed to get a SAX parser. Exception: " + e);
	  }
	catch (SAXException e)
	  {
	  throw new XPBException("SAXProcessor:: SAX error. Exception: " + e);
	  }

    StatCollector statCollector = new StatCollector(stat);

	try
	  {
	  p.parse(xmlFile, statCollector);
	  }
	catch (FileNotFoundException e)
	  {
	  throw new XPBException("SAXProcessor:: File not found. Exception: " + e);
	  }
	catch (SAXException e)
	  {
	  throw new XPBException("SAXProcessor:: SAX parsing error. Exception: " + e);
	  }
	catch (IOException e)
	  {
	  throw new XPBException("SAXProcessor:: I/O error. Exception: " + e);
	  }
	return stat;
    }
  public void   setup(HashMap map)
    {
	params = map;
	}
  }
