/*
 * Copyright (c) 2001 by Pankaj Kumar.  All Rights Reserved.
 *
 * This program is open source software; you may use, copy, modify, and
 * redistribute it under the terms of the LICENSE with which it was
 * originally distributed. The LICENSE or reference to the LICENSE
 * can be found in file LICENSE.txt in the base directory of distributed
 * software.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * LICENSE for more details.
 */
package org.xperf.xpb.xstat.sax;

import java.util.Stack;
import java.util.EmptyStackException;

import org.xperf.xpb.xstat.XMLStat;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class StatCollector extends DefaultHandler
  {
  private XMLStat stat = null;
  private Stack elstack = new Stack();

  public StatCollector(XMLStat stat)
    {
	this.stat = stat;
    }

  public void startElement(String uri, String localname, String name, Attributes attrs)
	{
	stat.addElem(name);

	try
	  {
	  String parent = (String)elstack.peek();
	  stat.incParentCount(name, parent);
      stat.incChildCount(parent, name);
	  }
	catch (EmptyStackException e)
	  {
	  stat.setRoot(name);
	  }

	// Attribute handling

	int n = attrs.getLength();
    for (int i = 0; i < n; i++)
      {
      String attnm = attrs.getQName(i);
      stat.incAttrCount(name, attnm);
      }

	elstack.push(name);
	}

  public void endElement(String uri, String localname, String name)
	    throws SAXException
	{
	try
	  {
	  elstack.pop();
	  }
	catch (EmptyStackException e)
	  {
	  throw new SAXException("Tried to pop empty element stack");
	  }
	}

  public void characters(char [] data, int offset, int len)
	    throws SAXException
	{
	String elname;
	try
	  {
	  elname = (String)elstack.peek();
	  }
    catch (EmptyStackException e)
      {
	  throw new SAXException("No element on stack");
	  }
    stat.addCharCount(elname, len);
	}

  public void ignorableWhitespace(char [] data, int offset, int len)
	    throws SAXException
	{
    String elname;
	try
	  {
	  elname = (String)elstack.peek();
      }
    catch (EmptyStackException e)
	  {
	  throw new SAXException("No element on stack");
	  }
    stat.addCharCount(elname, len);
	}
  }

